"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var path_1 = tslib_1.__importDefault(require("path"));
var plugins_1 = require("./plugins");
var mini_css_extract_plugin_1 = tslib_1.__importDefault(require("mini-css-extract-plugin"));
var REG_NODE_MODULES = /node_modules/;
function getThemeMap(packageMap, cwd) {
    var theme = {};
    var packageMapTheme = packageMap.theme;
    if (packageMapTheme && typeof packageMapTheme === 'string') {
        var pp = packageMapTheme;
        if (pp.charAt(0) === '.') {
            pp = path_1.default.resolve(cwd, pp);
        }
        var fn = require(pp);
        theme = fn();
    }
    else if (packageMapTheme && typeof packageMapTheme === 'object') {
        theme = packageMapTheme;
    }
    return theme;
}
function getLoaderMode(context) {
    var importPluginOptions = context.importPluginOptions;
    if (importPluginOptions && !Array.isArray(importPluginOptions)) {
        throw new Error('context.importOptions must be an array');
    }
    return commonLoaders(context);
}
function commonLoaders(context) {
    var babelOptions = context.babelOptions, tsOptions = context.tsOptions;
    var JSX_LOADER = [{ loader: 'babel-loader', options: babelOptions }];
    var TSX_LOADER = JSX_LOADER.concat([{ loader: 'ts-loader', options: tsOptions }]);
    return { JSX_LOADER: JSX_LOADER, TSX_LOADER: TSX_LOADER };
}
function mwRules(context, next) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var cwd, _a, limit, ENV, packageMap, hash, plugins, imagePublicPath, theme, postcssOptions, rules, _b, JSX_LOADER, TSX_LOADER, scriptRules, stylesRules, othersRules, filename, chunkFilename;
        return tslib_1.__generator(this, function (_c) {
            context.rules = [];
            next();
            cwd = context.cwd, _a = context.limit, limit = _a === void 0 ? 10240 : _a, ENV = context.ENV, packageMap = context.packageMap, hash = context.hash, plugins = context.plugins, imagePublicPath = context.imagePublicPath;
            theme = getThemeMap(packageMap, cwd);
            postcssOptions = context.postcssOptions, rules = context.rules;
            _b = getLoaderMode(context), JSX_LOADER = _b.JSX_LOADER, TSX_LOADER = _b.TSX_LOADER;
            scriptRules = [
                {
                    test: /\.worker\.jsx?$/,
                    exclude: [REG_NODE_MODULES],
                    use: [{ loader: 'workerize-loader' }].concat(JSX_LOADER)
                },
                {
                    test: /\.worker\.tsx?$/,
                    exclude: [REG_NODE_MODULES],
                    use: [{ loader: 'workerize-loader' }].concat(TSX_LOADER)
                },
                {
                    test: function (filePath) {
                        return /\.jsx?$/.test(filePath) && !/\.worker\.jsx?$/.test(filePath);
                    },
                    exclude: [REG_NODE_MODULES, /@babel(?:\/|\\{1,2})runtime/],
                    use: JSX_LOADER
                },
                {
                    test: function (filePath) {
                        return /\.tsx?$/.test(filePath) && !/\.worker\.tsx?$/.test(filePath);
                    },
                    exclude: [REG_NODE_MODULES],
                    use: TSX_LOADER
                },
                {
                    test: /\.mjsx?$/,
                    include: [REG_NODE_MODULES],
                    type: 'javascript/auto'
                },
                {
                    test: /\.(graphql|gql)$/,
                    exclude: [REG_NODE_MODULES],
                    use: [{ loader: 'graphql-tag/loader' }]
                }
            ];
            stylesRules = [
                {
                    test: function (filePath) {
                        return /\.css$/.test(filePath) && !/\.module\.css$/.test(filePath);
                    },
                    use: [
                        { loader: 'style-loader' },
                        {
                            loader: 'css-loader',
                            options: {
                                sourceMap: true
                            }
                        },
                        { loader: 'postcss-loader', options: postcssOptions }
                    ]
                },
                {
                    test: /\.module\.css$/,
                    use: [
                        { loader: 'style-loader' },
                        {
                            loader: 'css-loader',
                            options: {
                                sourceMap: true,
                                modules: {
                                    localIdentName: '[local]--[hash:base64:8]'
                                }
                            }
                        },
                        { loader: 'postcss-loader', options: postcssOptions }
                    ]
                },
                {
                    test: function (filePath) {
                        return /\.less$/.test(filePath) && !/\.module\.less$/.test(filePath);
                    },
                    use: [
                        { loader: 'style-loader' },
                        {
                            loader: 'css-loader',
                            options: {
                                sourceMap: true
                            }
                        },
                        { loader: 'postcss-loader', options: postcssOptions },
                        {
                            loader: 'less-loader',
                            options: {
                                javascriptEnabled: true,
                                sourceMap: true,
                                modifyVars: theme
                            }
                        }
                    ]
                },
                {
                    test: /\.module\.less$/,
                    use: [
                        { loader: 'style-loader' },
                        {
                            loader: 'css-loader',
                            options: {
                                sourceMap: true,
                                modules: {
                                    localIdentName: '[local]--[hash:base64:8]'
                                }
                            }
                        },
                        { loader: 'postcss-loader', options: postcssOptions },
                        {
                            loader: 'less-loader',
                            options: {
                                javascriptEnabled: true,
                                sourceMap: true,
                                modifyVars: theme
                            }
                        }
                    ]
                }
            ];
            othersRules = [
                {
                    test: /\.svgx$/,
                    use: [{ loader: '@svgr/webpack' }]
                },
                {
                    test: /\.json5$/,
                    use: [{ loader: 'json5-loader' }]
                },
                {
                    test: /\.(woff|woff2)?(\?v=\d+\.\d+\.\d+)?$/,
                    use: [
                        {
                            loader: 'file-loader',
                            options: { limit: limit, mimetype: 'application/font-woff' }
                        }
                    ]
                },
                {
                    test: /\.ttf(\?v=\d+\.\d+\.\d+)?$/,
                    use: [
                        {
                            loader: 'file-loader',
                            options: { limit: limit, mimetype: 'application/octet-stream' }
                        }
                    ]
                },
                {
                    test: /\.eot(\?v=\d+\.\d+\.\d+)?$/,
                    use: [
                        {
                            loader: 'file-loader',
                            options: { limit: limit, mimetype: 'application/vnd.ms-fontobject' }
                        }
                    ]
                },
                {
                    test: /\.svg(\?v=\d+\.\d+\.\d+)?$/,
                    use: [
                        {
                            loader: 'file-loader',
                            options: { limit: limit, mimetype: 'image/svg+xml' }
                        }
                    ]
                },
                {
                    test: /\.(bmp|png|jpe?g|gif)(\?v=\d+\.\d+\.\d+)?$/i,
                    use: [
                        {
                            loader: 'url-loader',
                            options: {
                                limit: limit,
                                name: 'static/media/[name].[hash:8].[ext]',
                                publicPath: imagePublicPath ? imagePublicPath : undefined
                            }
                        }
                    ]
                },
                {
                    test: /\.html?$/,
                    use: [
                        {
                            loader: 'file-loader',
                            options: {
                                name: '[path]-[name].[ext]'
                            }
                        }
                    ]
                },
                {
                    test: /\.hbs?$/,
                    use: [{ loader: 'mustache-loader' }]
                }
            ];
            if (ENV.isProduction || ENV.isBeta) {
                stylesRules.map(function (rule) { return (rule.use[0] = mini_css_extract_plugin_1.default.loader); });
                filename = hash ? '[name]-[contenthash:8].css' : '[name].css';
                chunkFilename = hash ? '[name]-[contenthash:8].chunk.css' : '[name].chunk.css';
                plugins.push(new mini_css_extract_plugin_1.default({ filename: filename, chunkFilename: chunkFilename }));
            }
            context.rules = scriptRules
                .concat([{ parser: { requireEnsure: false } }])
                .concat(stylesRules)
                .concat(othersRules)
                .concat(rules);
            plugins.push(new plugins_1.WatchIgnorePlugin([/\.d\.ts$/]));
            return [2];
        });
    });
}
exports.default = mwRules;
