"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var path_1 = tslib_1.__importDefault(require("path"));
var fs_1 = tslib_1.__importDefault(require("fs"));
var mwBuild_1 = tslib_1.__importDefault(require("./mwBuild"));
var mwBabelOptions_1 = tslib_1.__importDefault(require("./mwBabelOptions"));
var mwPostCSSOptions_1 = tslib_1.__importDefault(require("./mwPostCSSOptions"));
var mwTSOptions_1 = tslib_1.__importDefault(require("./mwTSOptions"));
var mwDevServer_1 = tslib_1.__importDefault(require("./mwDevServer"));
var mwMultiEntryHTML_1 = tslib_1.__importDefault(require("./mwMultiEntryHTML"));
var mwPlugins_1 = tslib_1.__importDefault(require("./mwPlugins"));
var mwRules_1 = tslib_1.__importDefault(require("./mwRules"));
var mwENV_1 = tslib_1.__importDefault(require("./mwENV"));
var mwDLL_1 = tslib_1.__importDefault(require("./mwDLL"));
var mwPWA_1 = tslib_1.__importDefault(require("./mwPWA"));
var mwCDN_1 = tslib_1.__importDefault(require("./mwCDN"));
var webpack_merge_1 = tslib_1.__importDefault(require("webpack-merge"));
function noop() {
    return tslib_1.__awaiter(this, void 0, void 0, function () { return tslib_1.__generator(this, function (_a) {
        return [2];
    }); });
}
function getCustomConfig(cwd, config) {
    var _this = this;
    if ('function' === typeof config) {
        return config;
    }
    var paths = [];
    switch (process.env.NODE_ENV) {
        case 'production':
            paths.push.apply(paths, ['build', 'production', 'prod']);
            break;
        case 'development':
            paths.push.apply(paths, ['development', 'develop', 'dev']);
            break;
        case 'beta':
            paths.push.apply(paths, ['beta', 'build', 'production', 'prod', 'development', 'develop', 'dev']);
            break;
        default:
            paths.push(process.env.NODE_ENV);
            break;
    }
    paths.push('config');
    paths = paths.map(function (name) { return "webpack." + name + ".js"; });
    config && paths.unshift(config);
    var cc = noop;
    var i = -1;
    while (++i < paths.length) {
        var p = paths[i];
        var pp = path_1.default.join(cwd, p);
        if (fs_1.default.existsSync(pp)) {
            console.info("" + pp);
            cc = require(pp);
            break;
        }
    }
    if ('function' === typeof cc) {
        return cc;
    }
    if ('object' === typeof cc) {
        return function (ctx) { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                ctx.webpackConfig = webpack_merge_1.default(ctx.webpackConfig, cc);
                if (cc.target && cc.output.libraryTarget) {
                    ctx.__IS_BUILD_LIBRARY__ = true;
                }
                return [2];
            });
        }); };
    }
    return null;
}
function mwsBuild(cwd, config) {
    var mwConfig = getCustomConfig(cwd, config);
    return [
        mwENV_1.default,
        mwBuild_1.default,
        mwPWA_1.default,
        mwMultiEntryHTML_1.default,
        mwDLL_1.default,
        mwPlugins_1.default,
        mwRules_1.default,
        mwBabelOptions_1.default,
        mwPostCSSOptions_1.default,
        mwTSOptions_1.default,
        mwConfig
    ];
}
exports.mwsBuild = mwsBuild;
function mwsDevServer(cwd, config) {
    var mwConfig = getCustomConfig(cwd, config);
    return [
        mwENV_1.default,
        mwBuild_1.default,
        mwPWA_1.default,
        mwCDN_1.default,
        mwMultiEntryHTML_1.default,
        mwDLL_1.default,
        mwPlugins_1.default,
        mwRules_1.default,
        mwBabelOptions_1.default,
        mwPostCSSOptions_1.default,
        mwTSOptions_1.default,
        mwDevServer_1.default,
        mwConfig
    ];
}
exports.mwsDevServer = mwsDevServer;
function mwsDLL(cwd, config) {
    var mwConfig = getCustomConfig(cwd, config);
    return [mwENV_1.default, mwBuild_1.default, mwDLL_1.default, mwPlugins_1.default, mwRules_1.default, mwBabelOptions_1.default, mwPostCSSOptions_1.default, mwTSOptions_1.default, mwConfig];
}
exports.mwsDLL = mwsDLL;
