"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var node_notifier_1 = tslib_1.__importDefault(require("node-notifier"));
exports.notifier = node_notifier_1.default;
var chalk_1 = tslib_1.__importDefault(require("chalk"));
exports.chalk = chalk_1.default;
var fs_1 = tslib_1.__importDefault(require("fs"));
var path_1 = tslib_1.__importDefault(require("path"));
function getProgressHandler(percent, msg1, msg2) {
    var stream = process.stdout;
    if (stream.isTTY && percent < 0.7) {
        stream.cursorTo(0);
        stream.write("\u231B  " + chalk_1.default.magenta(msg2) + " " + msg1);
        stream.clearLine(1);
    }
    else if (percent >= 1) {
        console.log(chalk_1.default.green('\nwebpack: bundle build is now finished'));
    }
}
exports.getProgressHandler = getProgressHandler;
function checkWebpackConfig(webpackConfig) {
    var configs = Array.isArray(webpackConfig) ? webpackConfig : [webpackConfig];
    var hasEmptyEntry = configs.some(function (c) { return Object.keys(c.entry || {}).length === 0; });
    if (hasEmptyEntry) {
        var e = new Error('no webpack entry found');
        e.name = 'NoEntry';
        throw e;
    }
}
exports.checkWebpackConfig = checkWebpackConfig;
function getValueByPath(path) {
    return !fs_1.default.existsSync(path) ? {} : require(path);
}
exports.getValueByPath = getValueByPath;
function getBuildCopyFiles(files) {
    if (Array.isArray(files)) {
        return files.map(function (from) { return ({ from: from }); });
    }
    return Object.keys(files)
        .filter(function (key) {
        var file = files[key];
        return typeof file === 'string' || !!file.path;
    })
        .map(function (key) {
        var file = files[key];
        if (typeof file === 'string') {
            return { from: file, to: key };
        }
        return { from: file.path, to: file.to || key };
    });
}
exports.getBuildCopyFiles = getBuildCopyFiles;
function getBuildExternals(files) {
    var ret = {};
    Object.keys(files).forEach(function (key) {
        var file = files[key];
        if (typeof file === 'string') {
            ret[key] = file;
        }
        else if (file.name) {
            ret[key] = file.name;
        }
    });
    return ret;
}
exports.getBuildExternals = getBuildExternals;
function getBuild4DevelopmentENV(filesMap) {
    var ret = [];
    Object.keys(filesMap).forEach(function (key) {
        var line = filesMap[key];
        var path = line.path;
        if (line.name && path) {
            ret.push(path);
        }
    });
    return ret;
}
exports.getBuild4DevelopmentENV = getBuild4DevelopmentENV;
function getBuild4ProductionENV(filesMap) {
    var ret = [];
    Object.keys(filesMap).forEach(function (key) {
        var line = filesMap[key];
        var path = line.path;
        if (line.name && path) {
            var paths = path.split('/');
            path = paths[paths.length - 1];
            ret.push(path);
        }
    });
    return ret;
}
exports.getBuild4ProductionENV = getBuild4ProductionENV;
function getBuildHTMLData(filesMap, env) {
    switch (env) {
        case 'production':
        case 'beta':
            return getBuild4ProductionENV(filesMap);
        default:
            return getBuild4DevelopmentENV(filesMap);
    }
}
exports.getBuildHTMLData = getBuildHTMLData;
function getBuildHTML(context) {
    var _a = context.externals, externals = _a === void 0 ? {} : _a, _b = context.sdks, sdks = _b === void 0 ? {} : _b, DLL_FILENAME = context.DLL_FILENAME, ENV = context.ENV, htmlWebpackPluginOptions = context.htmlWebpackPluginOptions;
    var entry = context.entry || context.packageMap.entry || getExampleEntry(context);
    if (!entry) {
        throw new Error('entry is an invalid value');
    }
    if (typeof entry === 'string') {
        context.entry = entry = { index: entry };
    }
    var paths = getBuildHTMLData(externals, ENV.env);
    var entryNames = Object.keys(entry);
    var options = tslib_1.__assign({ chunksSortMode: 'dependency', template: path_1.default.join(__dirname, '../assets/index.hbs'), favicon: path_1.default.join(__dirname, '../assets/favicon.ico') }, htmlWebpackPluginOptions);
    return entryNames.map(function (entryName) {
        var sdk = sdks[entryName];
        if (typeof sdk === 'string') {
            sdk = [sdk];
        }
        var scripts = paths.slice(0);
        if (sdk) {
            scripts.push.apply(scripts, sdk);
        }
        if (DLL_FILENAME) {
            scripts.push(DLL_FILENAME);
        }
        return tslib_1.__assign({ PATHS: scripts, scripts: scripts, entryName: entryName, cdnModule: entryName, filename: entryName + ".html", chunks: [entryName], inject: true, templateParameters: getBuildTemplateParametersWithScripts(scripts) }, options);
    });
}
exports.getBuildHTML = getBuildHTML;
function createDomain(_a) {
    var host = _a.host, port = _a.port;
    return "http://" + host + ":" + port;
}
exports.createDomain = createDomain;
function getNodeVersion(packageMap) {
    var emptyBuildIns = ['child_process', 'cluster', 'dgram', 'dns', 'fs', 'module', 'net', 'readline', 'repl', 'tls'];
    var browser = packageMap.browser || {};
    return emptyBuildIns.reduce(function (obj, name) {
        var _a;
        if (!(name in browser)) {
            return tslib_1.__assign({}, obj, (_a = {}, _a[name] = 'empty', _a));
        }
        return obj;
    }, {});
}
exports.getNodeVersion = getNodeVersion;
function getBuildSourceMap(devtool, ENV) {
    if (devtool === void 0) { devtool = false; }
    if (devtool === true) {
        devtool = ENV.isProduction ? false : '#cheap-module-eval-source-map';
    }
    return devtool;
}
exports.getBuildSourceMap = getBuildSourceMap;
function getBuildTemplateParametersWithScripts(scripts) {
    return function (compilation, assets, options) {
        var entryName = options.entryName;
        var stats = compilation.getStats().toJson();
        var currentAssets = stats.entrypoints[entryName].assets;
        var js = currentAssets.filter(function (n) { return path_1.default.extname(n) === '.js'; }).map(function (a) { return fixPublicPath(compilation.options, a); });
        var css = currentAssets.filter(function (n) { return path_1.default.extname(n) === '.css'; }).map(function (a) { return fixPublicPath(compilation.options, a); });
        var scripts2 = scripts.map(function (a) { return fixPublicPath(compilation.options, a); });
        assets.js = unique(scripts2.concat(assets.js).concat(js));
        assets.css = unique(assets.css.concat(css));
        return {
            compilation: compilation,
            webpack: compilation.getStats().toJson(),
            webpackConfig: compilation.options,
            htmlWebpackPlugin: {
                files: assets,
                options: options,
                scripts: js,
                styles: css,
                cdnModule: entryName
            }
        };
    };
}
exports.getBuildTemplateParametersWithScripts = getBuildTemplateParametersWithScripts;
function unique(array) {
    return array.filter(function (item, index, array) { return array.indexOf(item) === index; });
}
function getOptions(options, defaultOptions) {
    if (defaultOptions === void 0) { defaultOptions = {}; }
    return options === true ? defaultOptions : options;
}
exports.getOptions = getOptions;
function PromiseDefer() {
    var resolve = void 0;
    var reject = void 0;
    var promise = new Promise(function (rs, rj) {
        resolve = rs;
        reject = rj;
    });
    return { promise: promise, resolve: resolve, reject: reject };
}
exports.PromiseDefer = PromiseDefer;
function getExampleEntry(context) {
    if (!context.ENV.isDevelopment) {
        return null;
    }
    var entry = null;
    ['js', 'jsx', 'ts', 'tsx'].forEach(function (ext) {
        if (fs_1.default.existsSync(path_1.default.join(context.DIRs.root, 'example', "index." + ext))) {
            entry = "./example/index." + ext;
        }
    });
    return entry;
}
exports.getExampleEntry = getExampleEntry;
function fixPublicPath(options, url) {
    if (/(http|https):\/\/([\w.]+\/?)\S*/.test(url)) {
        return url;
    }
    var slash = '/';
    var output = options.output;
    if (output.publicPath.slice(-1) !== slash) {
        output.publicPath += slash;
    }
    var prefix = output.publicPath;
    if (prefix.slice(-1) !== slash) {
        prefix += slash;
    }
    return prefix + url;
}
exports.fixPublicPath = fixPublicPath;
